#include <stdbool.h>
#include <stdint.h>

// C_Meter Click
sbit C_METER_RST at RE1_bit;
sbit C_METER_RST_DIR at TRISE1_bit;
sbit C_METER_INT at RB0_bit;
sbit C_METER_INT_DIR at TRISB0_bit;

void system_setup( void );
void setup_interrupt( void );
void InitTimer0( void );

bool flag;
uint32_t start_val;
uint32_t end_val;
char uart_text[ 40 ];
char final_text[ 40 ];
uint32_t int_count = 0;
uint32_t timer_count_end = 0;
bool sec_flag = false;
float c_cal = 0.0;
float t_meas = 0.0;
float c_meas = 0.0;
float final = 0.0;
float t_cal = 0.0;


void main()
{
    //Local Declarations
    bool calibration_flag = true;


    system_setup();
    setup_interrupt();
    InitTimer0();

    t_cal = 1.0 / 22189.0;
    c_cal = t_cal / 77616.0;

    int_count = 0;

    while(1)
    {

        if( sec_flag )
        {
            //Debugging
            sprintf( uart_text, "Interrupts : %u\r\n", timer_count_end );
            UART1_Write_Text( uart_text );
            //Measurment
            t_meas = 1.0 / timer_count_end;
            c_meas = t_meas / 77616.0;
            final = fabs( c_cal - c_meas );
            //Print out
            UART1_Write_Text( "Calibrated: " );
            FloatToStr( c_cal, uart_text );
            UART1_Write_Text( uart_text );
            UART1_Write_Text( "\r\n" );
            UART1_Write_Text( "Final: " );
            FloatToStr( final, final_text );
            UART1_Write_Text( final_text );
            UART1_Write_Text( "\r\n" );
            //Reset flags
            sec_flag = false;
            int_count = 0;
            timer_count_end = 0;
        }
    }
}

void system_setup( void )
{
    //GPIO
    C_METER_INT_DIR = 1;
    C_METER_RST_DIR = 0;

    //RST Toggle
    C_METER_RST = 0;
    Delay_ms(50);
    C_METER_RST = 1;

    //UART
    UART1_Init( 9600 );
    UART1_Write_Text( "UART Initialized\r\n" );

}

void setup_interrupt( void )
{

  ADCON1 = 0x0F;           // AD converter off
  ANSELB = 0;
  TRISB = 0xFF;            // Set PB0 as input

  INTEDG0_bit = 0;         // Interrupt on falling edge on RB0
  INT0IF_bit = 0;          // Clear INT0IF
  INT0IE_bit = 1;          // turn OFF interrupt on INT0
  GIE_bit = 1;             // enable GIE
}

void InitTimer0(){
  T0CON         = 0x84;
  TMR0H         = 0x0B;
  TMR0L         = 0xDC;
  GIE_bit         = 1;
  TMR0IE_bit         = 1;
}

void Interrupt()
{
  if (INT0IF_bit && INT0IE_bit)
  {
     INT0IF_bit = 0;
     INT0IE_bit = 0;

     int_count++;
     INT0IE_bit = 1;
   }
   
  if (TMR0IF_bit)
  {
    TMR0IF_bit = 0;
    TMR0H         = 0x0B;
    TMR0L         = 0xDC;
    //Enter your code here
    timer_count_end = int_count;
    int_count = 0;
    sec_flag = true;
  }
  

}